@@ -195,7 +195,7 @@ def DQT(self, marker):
             raise SyntaxError("bad quantization table marker")
         v = i8(s[0])
         if v//16 == 0:
-            self.quantization[v & 15] = array.array("b", s[1:65])
+            self.quantization[v & 15] = array.array("B", s[1:65])
             s = s[65:]
         else:
             return  # FIXME: add code to read 16-bit tables!
@@ -331,6 +331,8 @@ class JpegImageFile(ImageFile.ImageFile):
             elif i == 0 or i == 0xFFFF:
                 # padded marker or junk; move on
                 s = b"\xff"
+            elif i == 0xFF00:  # Skip extraneous data (escaped 0xFF)
+                s = self.fp.read(1)
             else:
                 raise SyntaxError("no marker found")
 
@@ -399,7 +401,7 @@ def _fixup_dict(src_dict):
     # returns a dict with any single item tuples/lists as individual values
     def _fixup(value):
         try:
-            if len(value) == 1 and type(value) != type({}):
+            if len(value) == 1 and not isinstance(value, dict):
                 return value[0]
         except: pass
         return value
@@ -448,7 +450,7 @@ def _getexif(self):
         info = TiffImagePlugin.ImageFileDirectory_v1(head)
         info.load(file)
         exif[0x8825] = _fixup_dict(info)
-    
+
     return exif
 
 
@@ -640,7 +642,7 @@ def _save(im, fp, filename):
                 try:
                     if len(table) != 64:
                         raise
-                    table = array.array('b', table)
+                    table = array.array('B', table)
                 except TypeError:
                     raise ValueError("Invalid quantization table")
                 else:
